@define-color polari_dark_bg_color shade(@theme_bg_color, 0.95);

.info { margin-bottom: 1px; }

.polari-room-list .sidebar {
    border-style: solid;
    border-color: @borders;
    border-image: none;
}

.polari-room-list .sidebar:dir(ltr) {
    border-width: 0 1px 0 0;
}

.polari-room-list .sidebar:dir(rtl) {
    border-width: 0 0 0 1px;
}

.polari-user-list row.expanded {
    background-color: @content_view_bg;
}

.polari-user-list row frame {
    box-shadow: inset 0 2px 4px alpha(@polari_dark_bg_color, 0.8);
    transition: all 250ms ease-out;
    border-width: 0;
}

.polari-user-list .placeholder image {
    color: alpha(currentColor, 0.3);
}

.polari-user-list .placeholder label {
    font-size: 1.2em;
    color: alpha(currentColor, 0.6);
}

.polari-paste-confirmation {
    border: 0 solid @borders;
    border-top-width: 1px;
}

.polari-nick-button {
    padding-left: 8px;
    padding-right: 8px;
}

.polari-nick-entry {
    color: alpha(@theme_fg_color, 0.6);
}

.polari-room-list .room-list-header {
    font-size: smaller;
    padding: 8px 7px 0;
}

.polari-room-list .room-list-header label:dir(ltr),
.polari-room-list .room-list-header stack:dir(rtl) {
    padding-left: 8px;
}

.polari-room-list .room-list-header stack:dir(ltr),
.polari-room-list .room-list-header label:dir(rtl) {
    padding-right: 8px;
}

.polari-room-list .room-list-header separator {
    background-color: alpha(@borders, 0.55);
}

.polari-room-list row {
    padding-left: 8px;
}

.polari-room-list row.inactive *,
.polari-room-list row.inactive:backdrop * {
    color: mix(@theme_fg_color, @theme_base_color, 0.3);
}

.polari-room-list row.inactive:selected *,
.polari-room-list row.inactive:selected:backdrop * {
    color: mix(@theme_selected_fg_color, @theme_base_color, 0.3);
}

.polari-room-list row:backdrop * {
    color: @theme_fg_color;
}

.polari-room-list row:backdrop:selected * {
    color: @theme_selected_fg_color;
}

.polari-room-list row .pending-messages-count {
    color: @theme_selected_fg_color;
    background-color: mix(@theme_selected_bg_color, @theme_selected_fg_color, 0.3);
    font-size: smaller;
    border-radius: 4px;
    padding: 0px 4px;
}

.polari-room-list row .pending-messages-count:backdrop {
    color: @theme_unfocused_selected_fg_color;
    background-color: mix(@theme_unfocused_selected_bg_color, @theme_unfocused_selected_fg_color, 0.1);
}

treeview.polari-server-room-list {
    padding: 6px 12px;
}

.irc-feedback {
    color: @theme_fg_color;
    background-color: @theme_bg_color;
    border-style: solid;
    border-color: @borders;
    border-width: 1px 1px 0 1px;
    border-radius: 6px 6px 0 0;
    padding: 6px;
}

.url-preview { padding: 8px; }
.url-preview image { min-width: 120px; min-height: 90px; }
.url-preview label { font-size: small; }

.emoji-picker entry { margin: 6px; }

.emoji-picker .view { background-color: @theme_bg_color; }

.emoji-picker undershoot { border: 0 solid @borders; }
.emoji-picker undershoot:backdrop { border: 0 solid @unfocused_borders; }

.emoji-picker undershoot.bottom { border-bottom-width: 1px; }
.emoji-picker undershoot.top { border-top-width: 1px; }

.emoji widget { border-radius: 6px; }
.emoji label { padding: 6px; }

/* Hack: Move separator by 1px so that it aligns with the sidebar */
.titlebar > separator:dir(ltr) { margin-left: -1px; }
.titlebar > headerbar:first-child:dir(ltr) { margin-right: 1px; }
.titlebar > separator:dir(rtl) { margin-right: -1px; }
.titlebar > headerbar:first-child:dir(rtl) { margin-left: 1px; }

.polari-user-list-button {
    padding: 0px 4px;
}

.polari-background {
    opacity: 0.5;
}

.polari-background-title {
    font-size: 250%;
}

.polari-background-description {
    font-size: larger;
}

.polari-setup-page {
    padding: 48px 156px;
}

.polari-setup-title {
    font-size: 175%;
}

.polari-error-information {
    background-color: @polari_dark_bg_color;
    border-style: solid;
    border-color: @borders;
    border-width: 1px 0 0 0;
}
.polari-error-information GtkLabel {
    color: @theme_fg_color;
    font-size: smaller;
}

.polari-listbox-filterbar {
    border: solid @borders;
    border-width: 0 0 1px;
}

/* "opt-out" of insensitive view styling */
.polari-entry-area:disabled { background-color: @theme_base_color; }

/* the following adds a border on top of the inputbar when users scroll up */
.polari-chat-view > undershoot.bottom { border-bottom: 1px solid @borders; }

/* tweak labels in the user popover */
.polari-user-popover-nick { font-weight: bold; }
.polari-user-popover-status { font-size: smaller; }

.polari-room-loading {
    opacity: 0.3;
}

/* Differentiate test instance from "normal" ones */
window.test-instance headerbar:last-child {
    background-image: cross-fade(25% -gtk-icontheme('system-run-symbolic'));
    background-repeat: no-repeat;
    background-position: 1em center;
    background-size: 4em;
}

/* Differentiate snapshot builds from regular ones */
window.snapshot headerbar { background: none; }
window.snapshot .titlebar {
    background: linear-gradient(to left, #a5b1bd 0%, #a5b1bd 8%, @theme_bg_color 25%);
    box-shadow: inset 0 1px #f1f3f5;
    border-color: #909fae;
    color: alpha(@theme_fg_color, 0.4);
}
